classdef Channel
    %Channel Class to hold SPM data.
    %   Channel is a class that is used to store SPM data. This data used
    %   to be sored in a struct, but to enable operator overloading, this
    %   class has been constructed.
    
    properties
        % Channel name
        name = '?';
        % Channel unit
        unit = '1';
        % Channel display unit
        display_unit = '1';
        % Channel display scale
        display_scale = 1;
        % Channel type
        type = 'generalScanStandard';
        % Channel context
        context = {};
        % Channel session
        session = '';
        % Context reference index
        ref_index = -1;
        % Affine transform
        affine_transform = [];
        % Context reference index
        comment = '';
        % Data+time
        date_time = '0001-01-01T00:00:00.000000Z';
        % Control
        control = struct('name',{},'setting',struct('name',{},'value',{}));
        %control =
        %struct('name',{'b'},'setting',struct('name',{'b'},'value',{1}));
        % Parameter
        parameter = struct(...
            'name', '?', ...
            'start_value', struct('coarse', 0, 'fine', 0 , 'real', 0), ...
            'drift', 0, ...
            'shift', NaN, ...
            'unit', '1', ...
            'display_unit', '1', ...
            'display_scale', 1 ...
            );
        % Axis
        axis = struct(...
            'name', '?', ...
            'count', 1, ...
            'direction', '?', ...
            'interval', 0, ...
            'variable',  struct('ref_index', 1, 'extent', 0) ...
            );
        % Pixel space
        pixel_domain = 'spatial';
        %Pixel type
        pixel_type = 'real';
        % Pixels
        pixels = 1;
    end
    
    methods
        function obj = Channel(channel_struct)
            % Construct a Channel object from a channel struct.
            if nargin > 0
                fieldnames(channel_struct);
                fields = fieldnames(channel_struct);
                for i = 1:length(fields)
                    obj.(fields{i}) = channel_struct.(fields{i});
                end
            end
        end
        
        function sum = plus(obj1,obj2)
            % PLUS  Implement obj1 + obj2 for Channel. The channels obj1
            % and obj2 should have equal domains.
            if ~same_domain(obj1, obj2)
                me = MException('spm:Channel:plus:unequal_domains', ...
                    'Cannot add channels with unequal domains.');
                throw(me);
            end
            sum = Channel(obj1);
            sum.pixels = obj1.pixels + obj2.pixels;
        end
        
        function difference = minus(obj1,obj2)
            % MINUS  Implement obj1 - obj2 for Channel. The channels obj1
            % and obj2 should have equal domains.
            if ~same_domain(obj1, obj2)
                me = MException('spm:Channel:minus:unequal_domains', ...
                    'Cannot subtract channels with unequal domains.');
                throw(me);
            end
            difference = Channel(obj1);
            difference.pixels = obj1.pixels - obj2.pixels;
        end
        
        function product = times(obj1,obj2)
            % TIMES  Implement obj1 .* obj2 for Channel. The channels obj1
            % and obj2 should have equal domains.
            if ~same_domain(obj1, obj2)
                me = MException('spm:Channel:times:unequal_domains', ...
                    'Cannot multiply channels with unequal domains.');
                throw(me);
            end
            product = Channel(obj1);
            product.pixels = obj1.pixels .* obj2.pixels;
        end
        
        function quotient = rdivide(obj1,obj2)
            % RDIVIDE  Implement obj1 ./ obj2 for Channel. The channels
            % obj1 and obj2 should have equal domains.
            if ~same_domain(obj1, obj2)
                me = MException('spm:Channel:rdivide:unequal_domains', ...
                    'Cannot divide channels with unequal domains.');
                throw(me);
            end
            quotient = Channel(obj1);
            quotient.pixels = obj1.pixels ./ obj2.pixels;
        end
    end
end

