function channels_out = convert2si(channels_in)
%CONVERT2SI Conversion to S.I. units
%   CONVERT2SI(CHANNELS_IN) returns the S.I. unit converted version of the
%   imageScan channel array CHANNELS_IN. In case the conversion cannot be
%   applied due to unknown units, a warning is issued.

% Check input
try
    parser = inputParser;
    parser.addRequired('channels_in', @is_channel_array);
    parser.parse(channels_in)
catch error
    me = MException('spm:convert2si:input_error', 'Invalid input: %s', ...
        error.message);
    throw(me);
end

unit_list = {'millimeter', 'millimeters', 'mm', 'microns', 'micron', ...
    'um', 'm', 'nm', 'nanometer', 'nanometers', 'nanometre', ...
    'angstroem', 'angstroems', 'angstrom', 'angstroms'};

exp_list = {3, 3, 3, 6, 6, 6, 6, 9, 9, 9, 9, 12, 12, 12, 12};

exp_map = containers.Map(unit_list, exp_list);

channels_out = channels_in;

for k = 1:length(channels_out)
    for l = 1:length(channels_out(k).axis)
        for m = 1: length(channels_out(k).axis(l).variable)
            n = channels_out(k).axis(l).variable(m).ref_index;
            unit = channels_out(k).parameter(n).unit;
            if isKey(exp_map, unit)
                channels_out(k).axis(l).variable(m).extent = ...
                    10^-exp_map(unit) * ...
                    channels_out(k).axis(l).variable(m).extent;
            end
        end
    end
    for n = 1:length(channels_out(k).parameter)
        unit = channels_out(k).parameter(n).unit;
        if isKey(exp_map, unit)
            channels_out(k).parameter(n).start_value.coarse = ...
                10^-exp_map(unit) * ...
                channels_out(k).parameter(n).start_value.coarse;
            channels_out(k).parameter(n).start_value.fine = ...
                10^-exp_map(unit) * ...
                channels_out(k).parameter(n).start_value.fine;
            channels_out(k).parameter(n).start_value.real = ...
                10^-exp_map(unit) * ...
                channels_out(k).parameter(n).start_value.real;
            channels_out(k).parameter(n).drift = ...
                10^-exp_map(unit) * ...
                channels_out(k).parameter(n).drift;
            channels_out(k).parameter(n).shift = ...
                10^-exp_map(unit) * ...
                channels_out(k).parameter(n).shift;
            channels_out(k).parameter(n).unit = 'm';
            channels_out(k).parameter(n).display_scale = ...
                10^exp_map(unit) * ...
                channels_out(k).parameter(n).display_scale;
        end
    end
end

% Check whether units have been converted to m
for k=1:length(channels_out)
    for n = 1:length(channels_out(k).parameter)
        if ~strcmp(channels_out(k).parameter(n).unit, 'm')
            warning('spm:convert2si:no_conversion', ...
                'Parameter %s not converted to m.', ...
                channels_out(k).parameter(n).name);
        end
    end
end

end

