function channels = dom2channels(dom, context)
%DOM2CHANNELS Conversion of an EMPA XML DOM tree to an array of channels.
%   DOM2CHANNELS(DOM) is the channel array representation of an EMPA
%   XML DOM tree.

% Check whether dom is a DOM tree
if (~isa(dom, 'org.apache.xerces.dom.DocumentImpl') &&...
        ~isa(dom, 'org.apache.xerces.dom.DeferredDocumentImpl'))
    ME = MException('spm:dom2channels:invalid_dom', ...
        'First argument should be a DOM tree');
    throw(ME);
end

% if nargin == 1
%     context = '';
% end
if (nargin == 2 && ~ischar(context))
    ME = MException('spm:dom2channels:invalid_context', ...
        'First argument should be a DOM tree');
    throw(ME);
end
% Map to a structure
[~, ~, endian] = computer;
channel_list = xml_read(dom);

% Some correction to the structure
% Remove session if present
if (isfield(channel_list.channel,'session'))
    channel_list.channel = rmfield(channel_list.channel,'session');
end
for i = 1:length(channel_list.channel)
    % Map pixels to matrix
    uint8Data = base64decode(channel_list.channel(i).pixels);
    data = typecast(uint8Data, 'single');
    if endian == 'B'
        data = swapbytes(data);
    end
    data = double(data);
    width = channel_list.channel(i).axis(1).count;
    % Pure real-valued pixel data
    if strcmp(channel_list.channel(i).pixel_type, 'real')
        height = length(data(:)) / width;
        channel_list.channel(i).pixels = ...
            reshape(data(1:width*height), width, height)';
        % Complex pixel data
    elseif strcmp(channel_list.channel(i).pixel_type, 'complex')
        height = 0.5 * length(data(:)) / width;
        real = reshape(data(1:width*height), width, height)';
        imaginary = reshape(data(width*height+1:2*width*height), ...
            width, height)';
        channel_list.channel(i).pixels = complex(real, imaginary);
        % Unknown pixel data type
    else
        me = MException('spm:dom2channels:unknown_pixel_type', ...
            'Unknown pixel type.');
        throw(me);
    end
    % Overriding height in case of inconsistency with data vector length
    if length(channel_list.channel(i).axis) > 1
        if channel_list.channel(i).axis(2).count ~= height
            if (channel_list.channel(i).axis(2).variable.extent ~= 0)
                channel_list.channel(i).axis(2).variable.extent = ...
                    height / channel_list.channel(i).axis(2).count * ...
                    channel_list.channel(i).axis(2).variable.extent;
            end
            channel_list.channel(i).axis(2).count = height;
        end
    end
    
    % Overwrite context if given
    if nargin > 1
        if isempty(context) 
            channel_list.channel(i).context = {};
        else
            channel_list.channel(i).context = context;
        end
    end
    
    % Map single context to struct array
    if isfield(channel_list.channel(i),'context')
        if ~iscell(channel_list.channel(i).context)
            channel_list.channel(i).context =...
                cellstr(channel_list.channel(i).context);
        end
    end
    
    % Convert unit 1 to string (if necessary)
    if channel_list.channel(i).unit == 1
        channel_list.channel(i).unit = '1';
    end
    if channel_list.channel(i).display_unit == 1
        channel_list.channel(i).display_unit = '1';
    end
    for j = 1:length(channel_list.channel(i).parameter)
        if channel_list.channel(i).parameter(j).unit == 1
            channel_list.channel(i).parameter(j).unit = '1';
        end
        if channel_list.channel(i).parameter(j).display_unit == 1
            channel_list.channel(i).parameter(j).display_unit = '1';
        end
    end
    
    % Put fields in correct order
    full_path = mfilename('fullpath');
    root_path = fileparts(full_path);
    empa_skeleton_file = fullfile(root_path, '..', 'xml', ...
        'empa_skeleton.xml');
    empa_skeleton = xml_read(empa_skeleton_file);
    field_names = fieldnames(empa_skeleton.channel);
    example = empa_skeleton.channel;
    for j = 1 : length(field_names)
        if ~isfield(channel_list.channel, field_names(j))
            example = rmfield(example, field_names(j));
        end
    end
    channel_list.channel = orderfields(channel_list.channel,example);
end

% Convert channel structs to Channel onjects and put them in an array
nofchannels = length(channel_list.channel);
if nofchannels > 0
    channels(nofchannels) =  Channel(channel_list.channel(nofchannels));
    for i = 1: length(channel_list.channel)-1
        channels(i) = Channel(channel_list.channel(i));
    end
end

end

