function [hor_min, hor_max, vert_min, vert_max] = get_axes_limits(...
    channel, hor_axis_type, vert_axis_type)
%GET_AXES_LIMITS Helper function for spm_display to get axis limits.
%    [HOR_MIN, HOR_MAX, VERT_MIN,S VERT_MAX] = GET_AXES_LIMITS(CHANNEL,
%    HOR_AXIS_TYPE, VERT_AXIS_TYPE) returns the axis limits of the 
%    independent variable(s) of the channel CHANNEL given the axis type 
%    HOR_AXIS_TYPE and VERT_AXIS_TYPE for the first and second independent 
%    variable, respecitively. In case of a 1-dimensional channel the limits 
%    of the second axes are set to NaN.

% Check input
try
    parser = inputParser;
    parser.addRequired('channel', @is_channel);
    parser.addRequired('hor_axis_type', @(hor_axis_type)(ismember(...
        hor_axis_type,{'relative', 'fine', 'center', 'coarse', ...
        'real_fine', 'real_coarse', 'relative_real', 'none'})));
    parser.addRequired('vert_axis_type', @(vert_axis_type)(ismember(...
        vert_axis_type,{'relative', 'fine', 'center', 'coarse', ...
        'real_fine', 'real_coarse', 'relative_real', 'none'})));
    parser.parse(channel, hor_axis_type, vert_axis_type);
catch error
    me = MException('spm:get_axes_limits:input_error', ...
        'Invalid input: %s', error.message);
    throw(me);
end

% Search for axes and parameters
[hor_axis, vert_axis, hor_param, vert_param] = ...
    get_axes_and_params(channel);
% Get all horizontal parameter indices
hor_param_index = cell2mat({hor_axis.variable(:).ref_index});
% Compute horizontal extent
if length(hor_axis.variable) == 1
    hor_extent = hor_axis.variable.extent;
else
    hor_extent = norm(cell2mat({hor_axis.variable.extent}));
end

% Compute vertical extent (if applicable)
if ~isempty(vert_axis)
    if length(vert_axis.variable) == 1
        vert_extent = vert_axis.variable.extent;
    else
        vert_extent = norm(cell2mat({vert_axis.variable.extent}));
    end
    % Get all vertical parameter indices
    vert_param_index = cell2mat({vert_axis.variable(:).ref_index});
end

switch channel.pixel_domain
    case 'spatial'
        
        % Compute horizontal shift
        if length(hor_axis.variable) == 1
            hor_shift = hor_param.shift;
        else
            shift_vec = cell2mat(...
            {channel.parameter(hor_param_index).shift});
            extent_vec = cell2mat({hor_axis.variable.extent});
            in_product =  dot(shift_vec, extent_vec);
            norm_product = norm(shift_vec) * norm(extent_vec);
            if in_product == norm_product % Shift in extent direction
                hor_shift = norm(shift_vec);
            elseif in_product == -norm_product % Opposite to extent direction
                hor_shift = -norm(shift_vec);
            else % Off-axis shift
                hor_shift = NaN;
            end
        end
        
        % Compute vertical shift (if applicable)
        if ~isempty(vert_axis)
            if length(vert_axis.variable) == 1
                vert_shift = vert_param.shift;
            else
                shift_vec = cell2mat(...
            {channel.parameter(vert_param_index).shift});
                extent_vec = cell2mat({vert_axis.variable.extent});
                in_product =  dot(shift_vec, extent_vec);
                norm_product = norm(shift_vec) * norm(extent_vec);
                if in_product == norm_product % Shift in extent direction
                    vert_shift = norm(shift_vec);
                elseif in_product == -norm_product % Opposite to extent direction
                    vert_shift = -norm(shift_vec);
                else % Off-axis shift
                    vert_shift = NaN;
                end
            end
        end
        
        switch hor_axis_type
            case 'relative'
                hor_min = 0;
            case 'fine'
                hor_min = hor_param.start_value.fine * ...
                    hor_param.display_scale;
            case 'center'
                hor_min = -hor_extent * hor_param.display_scale / 2;
            case 'coarse'
                hor_min = (hor_param.start_value.coarse + ...
                    hor_param.start_value.fine) * hor_param.display_scale;
            case 'real_fine'
                hor_min = (hor_param.start_value.real - ...
                    hor_param.start_value.coarse) * ...
                    hor_param.display_scale;
            case 'real_coarse'
                hor_min = hor_param.start_value.real * ...
                    hor_param.display_scale;
            case 'relative_real'
                hor_min = hor_shift;
            case 'none'
                hor_min = NaN;
        end
        
        if hor_extent < 0
            hor_max = hor_min;
            hor_min = hor_max + hor_extent * hor_param.display_scale;
        else
            hor_max = hor_min + hor_extent * hor_param.display_scale;
        end
        
        if ~isempty(vert_axis)
            switch vert_axis_type
                case 'relative'
                    vert_min = 0;
                case 'fine'
                    vert_min = vert_param.start_value.fine * ...
                        vert_param.display_scale;
                case 'center'
                    vert_min = -vert_extent * vert_param.display_scale / 2;
                case 'coarse'
                    vert_min = (vert_param.start_value.coarse + ...
                        vert_param.start_value.fine) * ...
                        vert_param.display_scale;
                case 'real_fine'
                    vert_min = (vert_param.start_value.real - ...
                        vert_param.start_value.coarse) * ...
                        vert_param.display_scale;
                case 'real_coarse'
                    vert_min = vert_param.start_value.real * ...
                        vert_param.display_scale;
                case 'relative_real'
                    vert_min= vert_shift;
                case 'none'
                    vert_min = NaN;
            end
            if vert_extent < 0
                vert_max = vert_min;
                vert_min = vert_max + vert_extent * vert_param.display_scale;
            else
                vert_max = vert_min + vert_extent * vert_param.display_scale;
            end
        else
            vert_min = NaN;
            vert_max = NaN;
        end
    case 'frequency'
        hor_min = -hor_axis.count/ ...
            (2 * hor_extent  * hor_param.display_scale);
        hor_max = -hor_min;
        if ~isempty(vert_axis)
            vert_min = -vert_axis.count / ...
            (2 * vert_extent  * vert_param.display_scale );
            vert_max = -vert_min;
        else
            vert_min = NaN;
            vert_max = NaN;
        end
    otherwise
         me = MException('spm:get_axes_limits:unknown_pixel_domain', ...
                           'Unknown pixel space.');
        throw(me);
end
end


