function channels = sxm2channels(filename)
%SXM2CHANNELS Read an (EMPA) SXM file into an array of channels
%   SXM2CHANNELS(FILENAME) reads the EMPA SXM file FILENAME into a channel
%   array and returns this array. The function is a helper function for
%   SPM_READ.

sxm = read_sxm(filename);
if isempty(sxm.data)
    me = MException('spm:sxm2channels:unsupported_file', ...
        ['File type not supported. ', ...
        'Please provide a correct (EMPA) SXM file.']);
    throw(me);
end
two_dimensional =  (size(sxm.header.SCAN_LABEL, 1 ) > 1 && ...
    ~isempty(deblank(sxm.header.SCAN_LABEL(2, :))));
% Channel name
root_index = strfind(sxm.header.SCAN_FILE, '-> ');
if isempty(root_index)
    root_index = strfind(sxm.header.SCAN_FILE, '<- ');
end
if ~isempty(root_index)
    channel.name = sxm.header.SCAN_FILE(root_index+3:end);
else
    channel.name = '?';
end
% Channel unit
channel.unit = sxm.header.DATA_UVOLT;
% Channel display unit
channel.display_unit = sxm.header.DATA_UNIT;
% Channel display scale
channel.display_scale = sxm.header.DATA_TRANSFORM;
% Channel type
if two_dimensional &&...
        (strcmp(strtrim(sxm.header.SCAN_LABEL(1, :)), 'x') == 1 ||...
        strcmp(strtrim(sxm.header.SCAN_LABEL(1, :)), 'X') == 1) &&...
        (strcmp(strtrim(sxm.header.SCAN_LABEL(2, :)), 'y') == 1 ||...
        strcmp(strtrim(sxm.header.SCAN_LABEL(2, :)), 'Y') == 1)
    channel.type = 'imageScan';
else
    channel.type = 'generalScanStandard';
end
% Channel context
channel.context{1} = filename;
% Channel session
channel.session = '';
% Context reference index
channel.ref_index = -1;
% Channel affine transform
channel.affine_transform = [];
% Channel comment
channel.comment= '';

% Date+time
[~, name, ~] = fileparts(filename);
correct_backward = NaN;
if length(name) > 2
    if strcmp(name(end-1:end), '_f') == 1
        correct_backward = 0;
    elseif strcmp(name(end-1:end), '_b') == 1
        correct_backward =  sxm.header.SCAN_DTIME(1) * ...
            (size(sxm.data, 2) - 1);
    else
        correct_backward = NaN;
    end
end
correct_down = 0;
if strcmp(sxm.header.SCAN_DTIME(2), 'down')
    correct_down = sxm.header.SCAN_DTIME(2) * (size(sxm.data, 1) - 1);
end
to_add = correct_backward + correct_down + sxm.header.SCAN_TIME;
if ~isnan(to_add)
    prefix_date = addtodate(datenum(sxm.header.SCAN_DATE, 1), ...
        floor(to_add), 'second');
    tail_date = sprintf('%.6f', mod(to_add, 1));
    tail_date= ['.', tail_date(3:end), 'Z'];
    channel.date_time = [...
        strrep(datestr(prefix_date, 'yyyy-mm-dd HH:MM:SS'), ' ', ...
        'T'), tail_date];
else
    % Setting it to date-time that is practically impossible
    channel.date_time = '0001-01-01T00:00:00.000000Z';
end

% Control parameters
channel.control = struct('name',{},'setting',{});

% Parameter x
channel.parameter(1,1).name = 'x';
% Position
channel.parameter(1,1).start_value.coarse = NaN;
if ismember(sxm.header.SCAN_LABEL(1, :),{'X','x'})
    channel.parameter(1,1).start_value.fine = sxm.header.SCAN_OVOLT(1);
elseif  ismember(sxm.header.SCAN_LABEL(2, :),{'X','x'})
    channel.parameter(1,1).start_value.fine = sxm.header.SCAN_OVOLT(2);
else
    channel.parameter(1,1).start_value.fine = NaN;
end
channel.parameter(1,1).start_value.real = NaN;
% Drift
channel.parameter(1,1).drift = NaN;
% Shift
channel.parameter(1,1).shift = NaN;
% Unit
if ismember(sxm.header.SCAN_LABEL(1, :),{'X','x'})
    channel.parameter(1,1).unit =  sxm.header.SCAN_UVOLT(1, :);
elseif  ismember(sxm.header.SCAN_LABEL(2, :),{'X','x'})
    channel.parameter(1,1).unit =  sxm.header.SCAN_UVOLT(2, :);
else
    channel.parameter(1,1).unit = '?';
end
% Display unit
if ismember(sxm.header.SCAN_LABEL(1, :),{'X','x'})
    channel.parameter(1,1).display_unit =  sxm.header.SCAN_UNIT(1, :);
elseif  ismember(sxm.header.SCAN_LABEL(2, :),{'X','x'})
    channel.parameter(1,1).display_unit =  sxm.header.SCAN_UNIT(2, :);
else
    channel.parameter(1,1).display_unit =  '?';
end
% Display scale
if ismember(sxm.header.SCAN_LABEL(1, :),{'X','x'})
    channel.parameter(1,1).display_scale = ...
        sxm.header.SCAN_TRANSFORM(1, 1);
elseif  ismember(sxm.header.SCAN_LABEL(2, :),{'X','x'})
    channel.parameter(1,1).display_scale = ...
        sxm.header.SCAN_TRANSFORM(2, 2);
else
    channel.parameter(1,1).display_scale =  NaN;
end

% Parameter y
channel.parameter(2,1).name = 'y';
% Position
channel.parameter(2,1).start_value.coarse = NaN;
if ismember(sxm.header.SCAN_LABEL(1, :),{'Y','y'})
    channel.parameter(2,1).start_value.fine = sxm.header.SCAN_OVOLT(1);
elseif  ismember(sxm.header.SCAN_LABEL(2, :),{'Y','y'})
    channel.parameter(2,1).start_value.fine = sxm.header.SCAN_OVOLT(2);
else
    channel.parameter(2,1).start_value.fine = NaN;
end
channel.parameter(2,1).start_value.real = NaN;
% Drift
channel.parameter(2,1).drift = NaN;
% Shift
channel.parameter(2,1).shift = NaN;
% Unit
if ismember(sxm.header.SCAN_LABEL(1, :),{'Y','y'})
    channel.parameter(2,1).unit =  sxm.header.SCAN_UVOLT(1, :);
elseif  ismember(sxm.header.SCAN_LABEL(2, :),{'Y','y'})
    channel.parameter(2,1).unit =  sxm.header.SCAN_UVOLT(2, :);
else
    channel.parameter(2,1).unit = '?';
end
% Display unit
if ismember(sxm.header.SCAN_LABEL(1, :),{'Y','y'})
    channel.parameter(2,1).display_unit =  sxm.header.SCAN_UNIT(1, :);
elseif  ismember(sxm.header.SCAN_LABEL(2, :),{'Y','y'})
    channel.parameter(2,1).display_unit =  sxm.header.SCAN_UNIT(2, :);
else
    channel.parameter(2,1).display_unit =  '?';
end
% Display scale
if ismember(sxm.header.SCAN_LABEL(1, :),{'Y','y'})
    channel.parameter(2,1).display_scale = ...
        sxm.header.SCAN_TRANSFORM(1, 1);
elseif  ismember(sxm.header.SCAN_LABEL(2, :),{'Y','y'})
    channel.parameter(2,1).display_scale = ...
        sxm.header.SCAN_TRANSFORM(2, 2);
else
    channel.parameter(2,1).display_scale =  NaN;
end

% Parameter z_rel
channel.parameter(3,1).name = 'z_rel';
% Position
channel.parameter(3,1).start_value.coarse = 0;
channel.parameter(3,1).start_value.fine = 0;
channel.parameter(3,1).start_value.real = 0;
% Drift
channel.parameter(3,1).drift = NaN;
% Shift
channel.parameter(3,1).shift = NaN;
% Unit
if (ismember(sxm.header.SCAN_LABEL(1, :),{'X','x'}) || ...
        ismember(sxm.header.SCAN_LABEL(1, :),{'Y','y'}) ||...
        strcmp(sxm.header.SCAN_LABEL(1, :), 'Sample Fine(xy)'))
    channel.parameter(3,1).unit =  sxm.header.SCAN_UVOLT(1, :);
elseif  (ismember(sxm.header.SCAN_LABEL(2, :),{'X','x'}) || ...
        ismember(sxm.header.SCAN_LABEL(2, :),{'Y','y'}) ||...
        strcmp(sxm.header.SCAN_LABEL(2, :), 'Sample Fine(xy)'))
    channel.parameter(3,1).unit =  sxm.header.SCAN_UVOLT(2, :);
else
    channel.parameter(3,1).unit = '?';
end
% Display unit
if (ismember(sxm.header.SCAN_LABEL(1, :),{'X','x'}) || ...
        ismember(sxm.header.SCAN_LABEL(1, :),{'Y','y'}) ||...
        strcmp(sxm.header.SCAN_LABEL(1, :), 'Sample Fine(xy)'))
    channel.parameter(3,1).display_unit =  sxm.header.SCAN_UNIT(1, :);
elseif  (ismember(sxm.header.SCAN_LABEL(2, :),{'X','x'}) || ...
        ismember(sxm.header.SCAN_LABEL(2, :),{'Y','y'}) ||...
        strcmp(sxm.header.SCAN_LABEL(2, :), 'Sample Fine(xy)'))
    channel.parameter(3,1).display_unit =  sxm.header.SCAN_UNIT(2, :);
else
    channel.parameter(3,1).display_unit =  '?';
end
% Display scale
if (ismember(sxm.header.SCAN_LABEL(1, :),{'X','x'}) || ...
        ismember(sxm.header.SCAN_LABEL(1, :),{'Y','y'}) ||...
        strcmp(sxm.header.SCAN_LABEL(1, :), 'Sample Fine(xy)'))
    channel.parameter(3,1).display_scale = ...
        sxm.header.SCAN_TRANSFORM(1, 1);
elseif  (ismember(sxm.header.SCAN_LABEL(2, :),{'X','x'}) || ...
        ismember(sxm.header.SCAN_LABEL(2, :),{'Y','y'}) ||...
        strcmp(sxm.header.SCAN_LABEL(2, :), 'Sample Fine(xy)'))
    channel.parameter(3,1).display_scale = ...
        sxm.header.SCAN_TRANSFORM(2, 1);
else
    channel.parameter(1,1).display_unit =  NaN;
end

% Parameter corresponding to horizontal axis:
% Name
if (~ismember(sxm.header.SCAN_LABEL(1, :),{'X','x'}) && ...
        ~ismember(sxm.header.SCAN_LABEL(1, :),{'Y','y'}))
    n_hor = size(channel.parameter, 1) + 1;
    % Name
    channel.parameter(n_hor,1).name = sxm.header.SCAN_LABEL(1, :);
    % Position
    channel.parameter(n_hor,1).start_value.coarse = NaN;
    channel.parameter(n_hor,1).start_value.fine = ...
        sxm.header.SCAN_OVOLT(1);
    channel.parameter(n_hor,1).start_value.real = NaN;
    % Drift
    channel.parameter(n_hor,1).drift = NaN;
    % Shift
    channel.parameter(n_hor,1).shift = NaN;
    % Unit
    channel.parameter(n_hor,1).unit =  sxm.header.SCAN_UVOLT(1, :);
    % Display unit
    channel.parameter(n_hor,1).display_unit =  sxm.header.SCAN_UNIT(1, :);
    % Display scale
    channel.parameter(n_hor,1).display_scale = ...
        sxm.header.SCAN_TRANSFORM(1, 1);
elseif ismember(sxm.header.SCAN_LABEL(1, :),{'X','x'})
    n_hor = 1;
else
    n_hor = 2;
end

% Parameter corresponding to vertical axis if present:
if two_dimensional
    if (~ismember(sxm.header.SCAN_LABEL(2, :),{'X','x'}) && ...
            ~ismember(sxm.header.SCAN_LABEL(2, :),{'Y','y'}))
        n_ver = size(channel.parameter, 1) + 1;
        % Name
        channel.parameter(n_ver,1).name = sxm.header.SCAN_LABEL(2, :);
        % Position
        channel.parameter(n_ver,1).start_value.coarse = NaN;
        channel.parameter(n_ver,1).start_value.fine = ...
            sxm.header.SCAN_OVOLT(2);
        channel.parameter(n_ver,1).start_value.real = NaN;
        % Drift
        channel.parameter(n_ver,1).drift = NaN;
        % Shift
        channel.parameter(n_ver,1).shift = NaN;
        % Unit
        channel.parameter(n_ver,1).unit =  sxm.header.SCAN_UVOLT(2, :);
        % Display unit
        channel.parameter(n_ver,1).display_unit =  ...
            sxm.header.SCAN_UNIT(2, :);
        % Display scale
        channel.parameter(n_ver,1).display_scale = ...
            sxm.header.SCAN_TRANSFORM(2, 2);
    elseif ismember(sxm.header.SCAN_LABEL(2, :),{'X','x'})
        n_ver = 1;
    else
        n_ver = 2;
    end
end

% Horizontal axis:
% Name
channel.axis(1,1).name = channel.parameter(n_hor,1).name;
% Count
channel.axis(1,1).count = size(sxm.data, 2);
% Direction
if sxm.header.SCAN_DTIME(1) > 0
    channel.axis(1,1).direction = 'forward';
else
    channel.axis(1,1).direction = 'backward';
end
% Interval
channel.axis(1,1).interval = sxm.header.SCAN_DTIME(1);
% Variable
channel.axis(1,1).variable.ref_index = n_hor;
% Extent
channel.axis(1,1).variable.extent = sxm.header.SCAN_DVOLT(1) * ...
    channel.axis(1,1).count;

% Vertical axis if present:
if two_dimensional
    % Name
    channel.axis(2,1).name = channel.parameter(n_ver,1).name;
    % Count
    channel.axis(2,1).count = size(sxm.data, 1);
    % Direction
    if sxm.header.SCAN_DTIME(2) > 0
        channel.axis(2,1).direction = 'forward';
    else
        channel.axis(2,1).direction = 'backward';
    end
    % Interval
    channel.axis(2,1).interval = sxm.header.SCAN_DTIME(2);
    % Variable
    channel.axis(2,1).variable.ref_index = n_ver;
    % Extent
    channel.axis(2,1).variable.extent = sxm.header.SCAN_DVOLT(2) * ...
        channel.axis(2,1).count;
end

% Pixel space
channel.pixel_domain = 'spatial';
%Pixel type
channel.pixel_type = 'real';
% Pixels
channel.pixels = sxm.data;
% channel=orderfields(channel);
 channels(1) = Channel(channel);

end

