function dest = xslt_flex(source, stylesheet, engine, compiled, parameters)
%XSLT_FLEX Xalan/Saxon-based XSL transformation
%   XSLT_FLEX(SOURCE, STYLESHEET,  ENGINE, COMPILED, PARAMETERS) is the
%   transformation result of applying the style sheet STYLESHEET (
%   filename)to XML input SOURCE (filename or DOM node) applying
%   the parameters as defined in the  structure PARAMETERS using the XSLT
%   engine ENGINE ('xalan' or 'saxon'). The current engines are  Xalan
%   2.7.1. or Saxon 9.  The boolean flag COMPILED indicates whether the
%   stylesheet is in a compiled form (only supported for the Saxon engine).
%   By default COMPILED is set to false. If PARAMETERS is missing no
%   parameters are applied.

if nargin < 5
    parameters = struct([]);
    if nargin < 4
        compiled = false;
    end
end
% Define paths and file names
full_path = mfilename('fullpath');
root_path = fileparts(full_path);
xslt_path = [root_path, filesep, '..', filesep, 'external', filesep, ...
    'xalan-j_2_7_1'];

% Make sure that we are using the right XSLT engine
prop=java.lang.System.getProperties;
old_prop = prop.get('javax.xml.transform.TransformerFactory');
if strcmp(engine, 'xalan') == 1
    if compiled
        me = MException('spm:xslt_flex:compiled_xalan', ...
            'Combination Xalan + compiled is not supported.');
        throw(me);
    end
    factory = ['com.sun.org.apache.xalan.internal.xsltc.trax.', ...
        'TransformerFactoryImpl'];
elseif strcmp(engine, 'saxon') == 1
    factory = 'net.sf.saxon.TransformerFactoryImpl';
else
    me = MException('spm:xslt_flex:engine_invalid', ...
        'Third argument should be a supported XSLT engine.');
    throw(me);
end
prop.put('javax.xml.transform.TransformerFactory', factory);

added_xalan_to_path = false;
if strcmp(engine, 'xalan') == 1
    cp = javaclasspath;
    if ~ismember(xslt_path, cp)
        javaaddpath(xslt_path);
        added_xalan_to_path = true;
    end
end

% Make sure the XSLT engine is set back in its original state:
c = onCleanup(...
    @()revert_xslt(prop, old_prop, added_xalan_to_path, xslt_path));

% Define the XML and XSLT sources

if ischar(source) && exist(source, 'file')
    source = get_full_path(source);
    xml_source = javax.xml.transform.stream.StreamSource(source);
elseif (isa(source, 'org.apache.xerces.dom.DocumentImpl')||...
        isa(source, 'org.apache.xerces.dom.DeferredDocumentImpl'))
    xml_source = javax.xml.transform.dom.DOMSource(source);
else
    me = MException('spm:xslt_flex:source_invalid', ...
        'First argument should be a file or a DOM node');
    throw(me);
end

if ischar(stylesheet) &&  exist(stylesheet, 'file')
    stylesheet = get_full_path(stylesheet);
    xslt_source = javax.xml.transform.stream.StreamSource(stylesheet);
else
    me = MException('spm:xslt_flex:stylesheet_invalid', ...
        'Second argument should be a valid stylesheet');
    throw(me);
end

% Prepare output structure
result=javax.xml.transform.dom.DOMResult();

% Define the transformer
if ~compiled
    trans_fact = javax.xml.transform.TransformerFactory.newInstance(  );
    trans = trans_fact.newTransformer(xslt_source);
else
    config = net.sf.saxon.Configuration();
    prepared_stylesheet = ...
        net.sf.saxon.PreparedStylesheet.loadCompiledStylesheet(config, ...
        stylesheet);
    trans = prepared_stylesheet.newTransformer();
end

% Add parameters
param_fields = fieldnames(parameters);
for i = 1:length(param_fields)
    trans.setParameter(param_fields{i}, parameters.(param_fields{i}));
end

% Do the transformation
try
    trans.transform(xml_source, result);
catch e
    me = MException('spm:xslt_flex:transformation_failed', ...
        'Cannot apply transformation.\n %s', e.message);
    throw(me);
end
dest = result.getNode();

% Clean-up function
    function revert_xslt(prop, old_prop, added_xalan_to_path, xslt_path)
        prop.put('javax.xml.transform.TransformerFactory', old_prop);
        if added_xalan_to_path
            javarmpath(xslt_path);
        end
    end

end

