function out = mask_builder(figid)
%MASK_BUILDER Interactive mask building.
%   MASK_BUILDER(FIGID) returns the result of interactively drawing shapes
%   in the window FIGID, where FIGID can be a window name or a window
%   handle.
%
%   The type of shape is selected by pressing (and releasing)
%   short-cut keys:
%
%   1: rectangle
%   2: ellipse
%   3: circle
%   4: square
%   5: polygon
%
%   By default the shape is a rectangle.
%
%   Shapes are drawn using the mouse. The polygon is drawn by clicking
%   the consecutive corner points and double clicking when finished
%   (this will add another point and the close the shape). The other shapes
%   are drawn by dragging the mouse.
%
%   Shapes can be islands (white in resulting mask) or holes (black in
%   resulting mask). Islands and holes can be toggled by pressing and
%   releasing the minus key. During mask building, islands are depicted in
%   green and holes in red. Changing the color of the shapes effects their
%   status. If the color is different from green or red, the shape will not
%   turn up in thefinal mask.
%
%   The left bottom corner of the figure window shows the actual shape that
%   is to be drawn and indicates whether island/hole.
%
%   Shapes can be deleted in a last-in-first-out order by pressing
%   (and releasing) DEL or BACKSPACE. Another option is to (mis)use the
%   color changing option for this (via the context menu that appears when
%   right-clicking a shape).
%
%   To center the most recently drawn shape press and release 'c' or 'C'.
%
%   By default the entire image is a hole area, so the user is drawing on a
%   black plane. This means that holes get absorped into the background
%   unless an underlying island has been drawn first. To change this press
%   and release 'a' or 'A', then the entire image ('all') will be selected
%   as an island.
%
%   If the channel is in the frequency domain, the created mask is rotated
%   180 degrees and the combination of this image and the original mask is
%   returned. To combine the two images we take a union if the background
%   is a hole and an intersection otherwise.
%
%   To exit the function press ENTER or close the figure.

% Check input
try
    parser = inputParser;
    parser.addRequired('figid', @(figid) (length(figid) == 1 && ...
        (ishandle(figid) && ...
        ~isempty(findobj(figid,'type','figure'))) || ...
        (ischar(figid) && ...
        ~isempty(findobj('name', figid,'type','figure')))));
    parser.parse(figid);
catch error
    me = MException('spm:mask_builder:input_error', ...
        'Invalid input: %s', error.message);
    throw(me);
end

% Find figure
if ishandle(figid)
    fig = figid;
else
    fig = findobj('name', figid,'type','figure');
    if numel(fig) > 1
        fig = fig(1);
        warning('spm:mask_builder:ambiguous_identifier', ...
            'More than one window with indicated name');
    end
end

% Try to fetch channel data
channel = get(fig,'UserData');
if isempty(channel)
    me = MException('spm:mask_builder:no_channel_found', ...
        'Figure does not have a channel attached to it.');
    throw(me);
    
end

if ~is_channel(channel)
    me = MException('spm:mask_builder:invalid channel', ...
        'Figure does not contain a valid channel.');
    throw(me);
end

% Disable other tools
pan_state = get(pan(fig), 'Enable');
zoom_state = get(zoom(fig), 'Enable');
rotate3d_state = get(rotate3d(fig), 'Enable');
datacursormode_state = get(datacursormode(fig), 'Enable');
brush_state = get(brush(fig), 'Enable');

pan(fig, 'off');
zoom(fig, 'off');
rotate3d(fig,'off');
datacursormode(fig,'off');
brush(fig,'off');

plotedit(fig,'off');
set(fig,'MenuBar','none');
set(fig,'ToolBar','none')

% Activate figure
figure(fig);
close_fig = false;

% Look fr current axes
ax = gca;

% Determine center position
x_limits = xlim;
y_limits = ylim;
center_pos = [x_limits(1) + (x_limits(2) - x_limits(1)) / 2, ...
    y_limits(1) + (y_limits(2) - y_limits(1)) / 2];

% Define shape drawing functions
shape_mode = {'imrect','imellipse','im_circle','im_square','impoly'};
paint_mode = {'g','r'};
paint_label = {'island', 'hole'};
shapes = cell(100,1);
count = 0;
i_shape = 1;
i_paint = 1;
background = 'hole';
func = str2func(shape_mode{1});

% Get old callbacks and callback scheduling behavior
window_button_motion_fcn =  get(fig, 'WindowButtonMotionFcn');
window_button_up_fcn =  get(fig, 'WindowButtonUpFcn');
key_release_fcn =  get(fig, 'KeyReleaseFcn');
close_request_fcn = get(fig, 'CloseRequestFcn');
interruptible = get(fig,'Interruptible');
busy_action = get(fig,'BusyAction');

% Initialize shape variable to invalid handle
shape = func(ax,[0 0 0 0]);
set(shape, 'Visible', 'off');
delete(shape);

% Start shape drawing
status = uicontrol('Style','text', 'Units','normalize', ...
    'Position',[0,0,0.1,0.1],'BackgroundColor',get(fig,'Color'));
update_status();
start_new();

% Set new callbacks and scheduling behavior
interruptible = get(fig,'Interruptible');
busy_action = get(fig,'BusyAction');
set(fig,'KeyReleaseFcn',@switch_mode);
set(fig,'CloseRequestFcn',@close);
uiwait(fig);

    function move(~,~)
        try
            set(shape, 'Visible', 'on');
            pos_cursor = get(ax,'CurrentPoint');
            pos_cursor =  pos_cursor(1,1:2);
            pos_shape = shape.getPosition();
            pos_shape(1:2) = pos_cursor;
            if isvalid(shape)
                shape.setPosition(pos_shape);
            end
        catch er
            cleanup();
            rethrow(er);
        end
    end

    function set_cursor_and_callbacks()
        pos_cursor = get(ax,'CurrentPoint');
        pos_cursor = pos_cursor(1,1:2);
        if strcmp(func2str(func), 'impoly')
            shape = func(ax,pos_cursor);
            set(fig,'WindowButtonUpFcn',@add_point);
        else
            shape = func(ax,[pos_cursor 0 0]);
            set(fig,'WindowButtonUpFcn',@start_new);
        end
        set(fig,'WindowButtonMotionFcn',@move);
    end

    function start_new(~,~)
        try
            if isvalid(shape)
                shape.setPositionConstraintFcn(@(pos) shape.getPosition());
                shape.setColor(paint_mode{i_paint});
                count = count + 1;
                shapes{count} = shape;
            end
            set_cursor_and_callbacks();
        catch er
            cleanup();
            rethrow(er);
        end
    end

    function add_point(~,~)
        try
            set(fig,'WindowButtonMotionFcn','');
            selection_type = get(fig, 'SelectionType');
            if strcmp(selection_type,'open')
                % Close old polygon
                shape.setClosed(true);
                % Start new polygon
                start_new();
            else
                shape.setClosed(false);
                pos_shape = shape.getPosition();
                pos_cursor = get(ax,'CurrentPoint');
                pos_cursor =  pos_cursor(1,1:2);
                pos_shape = [pos_shape; pos_cursor];
                shape.setPosition(pos_shape);
            end
        catch er
            cleanup();
            rethrow(er);
        end
    end

    function update_status()
        try
            set(status,'String',[shape_mode{i_shape},' ', ...
                paint_label{i_paint}]);
            extent = get(status,'Extent');
            set(status,'Position', [0, 0, extent(3), extent(4)]);
        catch er
            cleanup();
            rethrow(er);
        end
    end

    function switch_mode(~,event)
        try
            switch (event.Character)
                case {'1', '2', '3', '4', '5'}
                    i_shape = str2double(event.Character);
                    func = str2func(shape_mode{i_shape});
                    % Hide and delete old shape
                    set(shape, 'Visible', 'off');
                    delete(shape);
                    update_status();
                    start_new();
                case{'a', 'A'}
                    background = 'island';
                    set(shape, 'Visible', 'off');
                    delete(shape);
                    shape = imrect(ax, ...
                        [x_limits(1), y_limits(1), x_limits(2) - x_limits(1), y_limits(2)- y_limits(1)], ...
                        'PositionConstraintFcn',  ...
                        @(pos)  [x_limits(1), y_limits(1), x_limits(2) - x_limits(1), y_limits(2)- y_limits(1)]);
                    shape.setColor(paint_mode{1}); %island
                    count = count + 1;
                    shapes{count} = shape;
                    %start_new();
                    %set(shape, 'Visible', 'on');
                    set_cursor_and_callbacks();
                case {'c', 'C'}
                    if count > 0
                        position = shapes{count}.getPosition();
                        if size(position, 2) == 4
                            position(1) = center_pos(1) - position(3) / 2;
                            position(2) = center_pos(2) - position(4) / 2;
                        else
                            min1 = min(position(:,1));
                            max1 = max(position(:,1));
                            min2 = min(position(:,2));
                            max2 = max(position(:,2));
                            shift = zeros(1,2);
                            shift(1) = center_pos(1) - (min1 + max1)/2;
                            shift(2) = center_pos(2) - (min2 + max2)/2;
                            shift = repmat(shift,size(position,1), 1);
                            position = position + shift;
                        end
                        shapes{count}.setPosition(position);
                    end
                case {'-'}
                    i_paint = mod(i_paint, 2) + 1;
                    update_status();
                otherwise
                    switch (event.Key)
                        case {'delete', 'backspace'}
                            if count > 0
                                if isvalid(shapes{count})
                                    set(shapes{count}, 'Visible', 'off');
                                    delete(shapes{count});
                                    count = count - 1;
                                end
                            end
                        case 'return'
                            create_mask();
                            cleanup();
                    end
            end
        catch er
            cleanup();
            rethrow(er);
        end
    end

    function create_mask()
        try
            % Create mask
            out = channel;
            out.unit = '1';
            out.display_unit = '1';
            out.display_scale = 1;
            out.name = '?';
            islands = zeros(size(channel.pixels));
            holes = zeros(size(channel.pixels));
            for i=1:count
                if isempty(find(shapes{i}.getColor() ~= [0 1 0], 1))
                    islands = shapes{i}.createMask() | islands;
                elseif isempty(find(shapes{i}.getColor() ~= ...
                        [1 0 0], 1))
                    holes = shapes{i}.createMask() |  holes;
                end
            end
            
            out.pixels = double(islands & ~holes);
            if strcmp(channel.pixel_domain, 'frequency')
                if strcmp(background, 'hole')
                    out.pixels = max(out.pixels, rot90(out.pixels,2));
                else
                    out.pixels = min(out.pixels, rot90(out.pixels,2));
                end
                out.pixels = circshift(out.pixels , ...
                    [ceil(size(out.pixels ,1)/2), ...
                    ceil(size(out.pixels ,2)/2)]);
            end
        catch er
            cleanup();
            rethrow(er);
        end
    end

    function cleanup()
        % Hide and delete all
        for j = 1:length(shapes)
            if ((isa(shapes{j}, 'imrect')  || ...
                    isa(shapes{j}, 'impoly')) && ...
                    isvalid(shapes{j}))
                set(shapes{j}, 'Visible', 'off');
                delete(shapes{j});
            end
        end
        set(shape, 'Visible', 'off');
        delete(shape);
        
        % Resume
        uiresume(fig);
        % Reset tools
        pan(fig, pan_state);
        zoom(fig, zoom_state);
        rotate3d(fig, rotate3d_state);
        datacursormode(fig, datacursormode_state);
        brush(fig, brush_state);
        set(fig,'MenuBar','figure');
        set(fig,'ToolBar','figure');
        % Reset callback scheduling behavior
        set(fig,'Interruptible', interruptible);
        set(fig,'BusyAction', busy_action);
        % Reset callbacks
        set(fig, 'WindowButtonMotionFcn', ...
            window_button_motion_fcn);
        set(fig, 'WindowButtonUpFcn', window_button_up_fcn);
        set(fig, 'KeyReleaseFcn',  key_release_fcn);
        set(fig, 'CloseRequestFcn',  close_request_fcn);
        % Remove status
        delete(status);
        
        % Close figure if necessary
        if close_fig
            delete(fig);
        end
    end

    function close(~,~)
        close_fig = true;
        create_mask();
        cleanup();
    end


end

