function out = mfm_calc_flat(in, z, thickness, delta_w, amplitude, ...
    derivative, transform, tf)
%MFM_CALC_FLAT MFM imaging simulation with a flat height/sample profile.
%   MFM_CALC_FLAT(IN, Z, THICKNESS, DELTA_W, AMPLITUDE, DERIVATIVE, 
%   TRANSFORM, TF) returns the simulation result of measuring channel IN 
%   with the tip described by TF, i.e.,
%
%   IN            Spatial channel of type imageScan that is used as input
%                 for the simulation. The input can be either of the 
%                 following cases: 
%                 1. magnetization pattern (preferably with 
%                    units A/m)
%                 2. magnetic moment per unit area pattern (preferably with
%                    units Am2/m2
%                 3. stray field or field derivative distribution 
%                    (preferably with units A/m resp. A/m^2)
%                 4. MFM measurement in any units
%
%   Z             Height distance between the data in
%                 from_channel to the data in the tf_channel,
%                 in case 1. this would be the distance to the upper
%                 sample surface. 
%
%   THICKNESS     THICKNESS = d
%                 a magnetization pattern of a sample with a thickness d 
%                 is considered. TF = 0.5 exp(-kz) [1-exp(-kd)] ...
%                 
%                 THICKNESS = Inf
%                 a magnetization pattern of a sample with infinite 
%                 thickness is considered. TF = 0.5 exp(-kz) ...
%                                 
%                 THICKNESS = 0
%                 an infinitesimally thin sheet of a magnetic moment per 
%                 area pattern is considered. TF = m/A 0.5 exp(-kz) ...
%                 
%                 THICKNESS = NaN
%                 use if no boundary condition exists, i.e. when NOT 
%                 starting from a M or m/A pattern.
%                 TF = input_field exp(-kz)
%
%   DELTA_W       Domain wall width
%
%   AMPLITUDE     Vibration amplitude, set AMPLITUDE = 0 to use
%                 the small amplitude approximation.
%                 
%                 NOTE: IF  AMPLITUDE NOT= 0 THEN use 
%                       z = tip-sample distance + A !
%   
%   DERIVATIVE    0: No change in units
%                 1: Calculate z-derivative
%
%   TRANSFORM     0: No coordinate transform
%                 1: Convert z-component to n-component (tf*LCF)
%                 2: Convert z-component to n-derivative
%                    (tf*LCF*LCF)
%
%                 NOTE: use TRANSFORM = 2 if 
%                      (DERIVATIVE = 1 AND transformation) is wanted
%              
%   TF            Channel that contains the instrument calibration function  
%                 of the tip.
%
% -------------------------------------------------------------------------
% NOTE: the units of all the distance parameters are assumed to be the same
%       as those of the IN pattern.
% -------------------------------------------------------------------------
              
% Check input
try
    parser = inputParser;
    parser.addRequired('in', @(in)is_channel(in) && ...
        strcmp(in.type,'imageScan') && ...
        strcmp(in.pixel_domain, 'spatial') && ...
        ~isempty(in.pixels));
    parser.addRequired('z', @isfloat);
    parser.addRequired('thickness', ...
          @(thickness)(isfloat(thickness) && thickness >= 0) || ...
          isnan(thickness));
    parser.addRequired('delta_w', @(delta_w)(isfloat(delta_w) && ...
	        delta_w >= 0) || isnan(delta_w));
    parser.addRequired('amplitude', @isfloat);
    parser.addRequired('derivative', @(derivative) derivative == 0 ||...
        derivative == 1);
    parser.addRequired('transform', @(transform) transform == 0 || ...
        transform == 1 ||  transform == 2);
    parser.addRequired('tf', @(tf)is_channel(tf) && ...
        strcmp(tf.type,'imageScan') && ...
        strcmp(tf.pixel_domain, 'frequency')  && ~isempty(tf.pixels));
     parser.parse(in, z, thickness, delta_w, amplitude, ...
     derivative, transform, tf);
catch error
    me = MException('spm:mfm_calc_flat:input_error', ...
        'Invalid input: %s', error.message);
    throw(me);
end

% Angle between the (upward) lever normal and the z-axis. (0 - 90 degrees)
theta = (12.0 / 180.0) * pi;
% Angle between the (upward) lever normal and the x-axis. (-180 - 180
% degrees)
phi = (-90.0 / 180.0) * pi;

s = size(in.pixels);
work_image = fft2(in.pixels)/(s(1) * s(2));

% Simulate effect of tip
mfm_tf_out = mfm_tf(s(2), compute_dxdy(in, 1), s(1), ...
 compute_dxdy(in, 2), tf, derivative, transform,  z, amplitude, theta, ...
 phi, thickness, delta_w);
work_image = work_image .* mfm_tf_out.pixels;
work_image(1, 1) = 0.0; % Result undefined in (1,1)

out = in;
out.pixels = real(ifft2(work_image) * s(1) * s(2));

% Set z_level
if strcmp(out.parameter(3).name, 'z_rel')
    out.parameter(3).start_value.fine = ...
        out.parameter(3).start_value.fine + z;
    out.parameter(3).start_value.real = ...
        out.parameter(3).start_value.real + z;
else
    me = MException('spm:mfm_calc_flat:Structural_error', ...
        'Parameter z_rel missing in input channel');
    throw(me);
end

end

