function [data, unit] = mouse_position(figid)
%MOUSE_POSITION Collect channel data points indicated by mouse clicks.
%  [DATA, UNIT] =  = MOUSE_POSITION(FIGID) collects clicked data points 
%  from the qMFM channel displayed in figure FIGID until an arbitrary key
%  mouse button is pressed. It then returns the selected points in a
%  structure of arrays DATA, where each coordinate system has a 
%  separate field (i.e., x.none, y.none, x.relative, y.relative, x.real, 
%  y.real etc.). The corresponding units are returned in the structure
%  UNIT, which contains thefields x, y and channel. Of course, the unit
%  does not apply to x.none and y.none whose unit is pixel by definition.
%
%  The identifier FIGID can be a figure handle (number as displayed in the
%  left upper corner of the figure) or a figure name. To apply the function
%  to the current figure, call MOUSE_POSITION(gcf).

% Check input
try
    parser = inputParser;
    parser.addRequired('figid', @(figid) (length(figid) == 1 && ...
        (ishandle(figid) && ...
        ~isempty(findobj(figid,'type','figure'))) || ...
        (ischar(figid) && ...
        ~isempty(findobj('name', figid,'type','figure')))));
    parser.parse(figid);
catch error
    me = MException('spm:mouse_position:input_error', ...
        'Invalid input: %s', error.message);
    throw(me);
end

% Find figure
if ishandle(figid)
    fig = figid;
else
    fig = findobj('name', figid,'type','figure');
    if numel(fig) > 1
        fig = fig(1);
        warning('spm:mouse_position:ambiguous_identifier', ...
            'More than one window with indicated name');
    end
end

% Try to fetch channel data
channel = get(fig,'UserData');
if isempty(channel)
    me = MException('spm:mouse_position:no_channel_found', ...
        'Figure does not have a channel attached to it.');
    throw(me);
end

ch_axes = findobj(fig,'Tag','channel_axes','type','axes');
if isempty(ch_axes)
    me = MException('spm:mouse_position:no channel_axes', ...
        'Figure does not have channel axes.');
    throw(me);
end

ch_image = findobj(fig,'Tag','channel_image','type','image');
if isempty(ch_image)
    me = MException('spm:mouse_position:no channel_image', ...
        'Figure does not have channel image.');
    throw(me);
    
end

if ~is_channel(channel)
    me = MException('spm:mouse_position:invalid channel', ...
        'Figure does not contain a valid channel.');
    throw(me);
end

% Disable other tools
pan_state = get(pan(fig), 'Enable');
zoom_state = get(zoom(fig), 'Enable');
rotate3d_state = get(rotate3d(fig), 'Enable');
datacursormode_state = get(datacursormode(fig), 'Enable');
brush_state = get(brush(fig), 'Enable');

pan(fig, 'off');
zoom(fig, 'off');
rotate3d(fig,'off');
datacursormode(fig,'off');
brush(fig,'off');

plotedit(fig,'off');
set(gcf,'MenuBar','none');
set(fig,'ToolBar','none');

% Activate figure
figure(fig);
%state = uisuspend(fig);

% Now take care of cross hair and coordinate tracking
info = impixelinfoval(fig,ch_image);

org_pointer = get(fig, 'Pointer');
org_keypress =  get(fig, 'KeyPressFcn');
set(fig,'KeyPressFcn',  @press);
set(fig, 'Pointer', 'fullcrosshair');
set(ch_image,'ButtonDownFcn',@click);
set(ch_image, 'UIContextmenu', [])
x_data = get(ch_image,'XData');
y_data = get(ch_image,'XData');
points = zeros(100,3);
count = 0;
data = [];
unit = [];
uiwait();

    function click(rc,eventdata)
        if ~strcmp(get(fig,'SelectionType'), 'alt')
            pos=get(ch_axes,'CurrentPoint');
            x = pos(1,1);
            y = pos(1,2);
            xi = round(x);
            yi = round(y);
            if ismember(xi, 1 : size(channel.pixels, 2)) && ...
                    ismember(yi, 1 : size(channel.pixels, 1))
                ci = channel.display_scale * impixel(channel.pixels,x,y);
                ci = ci(1);
                fprintf('You clicked ');
                fprintf('x:%10.4f, y:%10.4f, channel:%14e\n', x, y, ci);
                count = count + 1;
                points(count,:) = [x, y, ci];
            end
        else
            press(rc,eventdata);
        end
    end

    function press(~,~)
        % Calculate coordinates relative to various systems
        axis_types = {'none', 'relative', 'fine', 'center', 'coarse', ...
            'real_fine', 'real_coarse', 'relative_real'};

        for m = 1:length(axis_types)
            if ~strcmp(axis_types{m} ,'none')
                [hor_min, hor_max, vert_min, vert_max] = ...
                        get_axes_limits(channel, axis_types{m}, ...
                        axis_types{m});
                for n = 1:count
                    data.x.(axis_types{m})(n) = hor_min + ...
                        (points(n,1) - x_data(1)) * ...
                        (hor_max - hor_min)  / (x_data(2) - x_data(1));
                    data.y.(axis_types{m})(n) = vert_min + ...
                        (points(n,2) - y_data(1)) * ...
                        (vert_max - vert_min)  / (y_data(2) - y_data(1));
                end
            else
                for n = 1:count
                    data.x.(axis_types{m})(n) = round(points(n,1));
                    data.y.(axis_types{m})(n) = round(points(n,2));
                end
            end
        end
        fprintf('\n');
        fprintf('--------------------------------- \n');
        fprintf('x-pixel   y-pixel   channel-value \n');
        fprintf('--------------------------------- \n');
        for n = 1:count
            data.channel(n) = points(n, 3);
            fprintf('%7d %9d %15e\n',  data.x.('none')(n), ...
                data.y.('none')(n),data.channel(n) );
        end
        [~, ~, hor_param, vert_param] = get_axes_and_params(channel);
        unit.x = hor_param.display_unit;
        unit.y = vert_param.display_unit;
        unit.channel = channel.display_unit;
        %clean(fig, org_pointer, ch_image, info);
        set(fig, 'Pointer', org_pointer);
        set(fig, 'KeyPressFcn', org_keypress);
        set(ch_image,'ButtonDownFcn','');
        if ishandle(info)
            delete(info)
        end
        restore();
    end

    function restore()
        pan(fig, pan_state);
        zoom(fig, zoom_state);
        rotate3d(fig, rotate3d_state);
        datacursormode(fig, datacursormode_state);
        brush(fig, brush_state);
        set(fig,'MenuBar','figure');
        set(fig,'ToolBar','figure');
        set(fig, 'Pointer', org_pointer);
        set(ch_image,'ButtonDownFcn','');
        if ishandle(info)
            delete(info)
        end
        refresh(fig);
        uiresume();
        return;
    end

end

