function out = rotate90(in, n)
%ROTATE90 Rotation over multiples of 90 degrees.
%   ROTATE90(IN, N) returns the result of rotating imageScan IN over n*90 
%   degrees around its center in counter-clockwise direction (seen 
%   from the perspective of a coordinate system where upward and right
%   are the positive directions). The units of the dependent variables
%   should all be equal.

% Check input
try
    parser = inputParser;
    parser.addRequired('in', @(in)is_channel(in) && ...
        strcmp(in.type,'imageScan'));
    parser.addRequired('n', @(n) isnumeric(n) && numel(n) == 1 && ...
        floor(n) == n);
    parser.parse(in, n);
catch error
    me = MException('spm:rotate:input_error', ...
        'Invalid input: %s', error.message);
    throw(me);
end

[hor_axis, vert_axis, hor_param, vert_param] = ...
    get_axes_and_params(in);

if hor_param.unit ~= vert_param.unit
    me = MException('spm:rotate:unequal_units', ...
        'Units in both directions should be equal');
    throw(me);
end

% Rotate
out = in;
out.pixels = rot90(out.pixels, -n); % Minus added because of flipped y-axis

% Update axes and parameters
center_hor = hor_param.start_value.fine + ...
    0.5 * hor_axis.variable(1).extent;
center_vert = vert_param.start_value.fine + ...
    0.5 * vert_axis.variable(1).extent;
if mod(n,2) == 1
    i_hor =  hor_axis.variable(1).ref_index;
    i_vert =  vert_axis.variable(1).ref_index;
    
    out.parameter(i_hor).start_value.fine = center_hor - ...
        0.5 * vert_axis.variable(1).extent;
    out.parameter(i_vert).start_value.fine = center_vert - ...
        0.5 * hor_axis.variable(1).extent;
    
    diff_hor = out.parameter(i_hor).start_value.fine - ...
        in.parameter(i_hor).start_value.fine;
    diff_vert = out.parameter(i_vert).start_value.fine - ...
        in.parameter(i_vert).start_value.fine;
    
    out.parameter(i_hor).start_value.real =  diff_hor + ...
        out.parameter(i_hor).start_value.real;
    out.parameter(i_vert).start_value.real =  diff_vert + ...
        out.parameter(i_vert).start_value.real;
    
    out.axis(1).variable(1).extent =  vert_axis.variable(1).extent;
    out.axis(2).variable(1).extent =  hor_axis.variable(1).extent;
    
    out.axis(1).count =  vert_axis.count;
    out.axis(2).count =  hor_axis.count;
end

% Store affine transformation if a reference context has been indicated

if out.ref_index ~= -1
    center2origin = [[eye(2), [-center_hor; -center_vert]]; ...
        [zeros(1,2),1]];
    rotation90 = [0 -1 0; 1 0 0; 0 0 1];
    rotation = rotation90^mod(n,4);
    origin2center = [[eye(2), [center_hor; center_vert]]; ...
        [zeros(1,2),1]];
    out.affine_transform = origin2center * rotation * center2origin;
end




end

