function out = section(in, from_x, from_y, to_x, to_y, varargin)
%SECTION Cut out a line from a channel.
%   SECTION(IN, FROM_X, FROM_Y, TO_X, TO_Y, VARARGIN) returns a channel
%   containing the line in the spatial imageScan channel from
%   (FROM_X, FROM_Y) to (TO_X, TO_Y). The coordinate system, in which these 
%   start/end point coordinates are given, is indicated by a parameter 
%   'axis_type', which equals one of 'none', 'relative', 'center', 'fine',  
%   'coarse','real_fine', 'real_coarse'. By default axis_type is 'none',  
%   which implies that one actually indicates the rows/columns of the pixel 
%   matrix to define the begin/end points of the line.
%
%   The number of pixels in the constructed one-dimensional channel equals
%   fix(sqrt((TO_X_PIX - FROM_X_PIX)^2 + (TO_Y_PIX - FROM_Y_PIX)^2) + 1)
%   where (FROM_X_PIX, FROM_Y_PIX) to (TO_X_PIX, TO_Y_PIX) are the pixel
%   coordinates of the extracted line. The interpolation used is linear.
%
%   To ensure that the line starts and ends  on an existing pixel when 
%   using any of the other axis types, one can set the parameter 
%   'snap_to_grid' to true (by default it is false). So, for instance
%
%   SECTION(IN, LOWER_X, LOWER_Y, UPPER_X, UPPER_Y, 'axis_type', 'fine',
%   'snap_to_grid', 'true') 
%   
%   interprets the given start/end point coordinates in the fine coordinate
%   system, but snaps them to the pixel grid before extracting the line.
%
%   If the input channel is already displayed in a figure, it is also 
%   possible to call SECTION in an interactive mode. Instead of providing 
%   a channel and line coordinates, the user specifies a figure handle/name 
%   on the command line: SECTION(FIGID). The start and end point of 
%   the line are selected by clicking in the figure window.


% Check input
try
    parser = inputParser;
    parser.addRequired('in', @(in)(is_channel(in)  && ...
        strcmp(in.type,'imageScan') && ...
        strcmp(in.pixel_domain,'spatial')));
    parser.addRequired('from_x', @(from_x)(isnumeric(from_x) && ...
        numel(from_x) == 1));
    parser.addRequired('from_y', @(from_x)(isnumeric(from_y) && ...
        numel(from_y) == 1));
    parser.addRequired('to_x', @(from_x)(isnumeric(to_x) && ...
        numel(to_x) == 1));
    parser.addRequired('to_y', @(from_x)(isnumeric(to_y) && ...
        numel(to_y) == 1));
    parser.addParamValue('axis_type', 'none', @(axis_type)(...
        ischar(axis_type) && ismember(axis_type, {'relative', 'center', ...
        'fine', 'coarse', 'real_fine', 'real_coarse', 'relative_real', ...
        'none'})));
    parser.addParamValue('snap_to_grid', false, @islogical);
    parser.parse(in, from_x, from_y, to_x, to_y, varargin{:});
    axis_type = parser.Results.axis_type;
    snap_to_grid = parser.Results.snap_to_grid;
catch 
    try
        parser = inputParser;
        parser.addRequired('figid', @(figid) (length(figid) == 1 && ...
            (ishandle(figid) && ...
            ~isempty(findobj(figid,'type','figure'))) || ...
            (ischar(figid) && ...
            ~isempty(findobj('name', figid,'type','figure')))));
        parser.parse(in);
        figid = parser.Results.figid;
    catch error
        me = MException('spm:section:input_error', ...
            'Invalid input: %s', error.message);
        throw(me);
    end
end

% Interactive version
if nargin == 1
    % Find figure
    if ishandle(figid)
        fig = figid;
    else
        fig = findobj('name', figid,'type','figure');
        if numel(fig) > 1
            fig = fig(1);
            warning('spm:cut:ambiguous_identifier', ...
                'More than one window with indicated name');
        end
    end

    % Try to fetch channel data
    in = get(fig,'UserData');
    if isempty(in)
        me = MException('spm:cut:no_channel_found', ...
            'Figure does not have a channel attached to it.');
        throw(me);
    end
    
    figure(fig);
    [x, y] = ginput(2);
    from_x = x(1);
    to_x = x(2);
    from_y = y(1);
    to_y = y(2);
    
    axis_type = 'none';
    snap_to_grid = false;
end


% Copy input to output
out = in;

% Check whether boundaries are allowed
[hor_min, hor_max, vert_min, vert_max] = get_axes_limits(in, ...
    axis_type, axis_type);
if strcmp(axis_type, 'none')
    hor_min = 1;
    vert_min = 1;
    hor_max = size(in.pixels,2);
    vert_max = size(in.pixels,1);
end
if to_x > hor_max || from_x < hor_min || to_y > vert_max || ...
        from_y < vert_min
    me = MException('spm:section:line_out_of_range', ...
        'Selected line is outside channel frame.');
    throw(me);
end

% Define cut-out rectangle
extent_x = in.axis(1).variable.extent;
extent_y = in.axis(2).variable.extent;
[ny, nx] = size(in.pixels);
ix =  out.axis(1).variable.ref_index;
iy =  out.axis(2).variable.ref_index;
scale_x =  out.parameter(ix).display_scale;
scale_y =  out.parameter(iy).display_scale;

if extent_x == 0 || extent_y == 0
    me = MException('spm:section:zero_extent', ...
        'Zero extent (is not supported).');
    throw(me);
end
if snap_to_grid && ~strcmp(axis_type,'none')
    from_x = fix(1 + (nx - 1) * ...
        (from_x - hor_min) / (abs(extent_x) * scale_x));
    from_y = fix(1 + (ny - 1) * ...
        (from_y - vert_min) / (abs(extent_y) * scale_y));
    to_x = fix(1 + (nx - 1) * ...
        (to_x - hor_min) / (abs(extent_x) * scale_x));
    to_y = fix(1 + (ny - 1) * ...
        (to_y - vert_min) / (abs(extent_y) * scale_y));
    axis_type = 'none';
    hor_min = 1;
    vert_min = 1;
end

if to_x == from_x && to_y == from_y
    me = MException('spm:section:no_line', ...
        'Points do not define line.');
    throw(me);
end

% Select the line
nof_points = fix(sqrt( (to_x - from_x)^2 + (to_y - from_y)^2 ) + 1);
 if nx > 1  && ny > 1% Otherwise something strange is going on, but
        % we are allowing empty channels for now
step_x_org = (hor_max - hor_min) / (nx - 1);
step_y_org = (vert_max - vert_min) / (ny - 1);
x_org = hor_min : step_x_org : hor_max;
y_org = vert_min : step_y_org : vert_max;
[xx_org, yy_org] = meshgrid(x_org, y_org);
step_x_new = (to_x - from_x) / (nof_points - 1);
step_y_new = (to_y - from_y) / (nof_points - 1);
if step_x_new ~=0 
    x_new = from_x : step_x_new : to_x;
else
    x_new = from_x * ones(1, nof_points);
end
if step_y_new ~=0 
    y_new = from_y : step_y_new : to_y;
else
    y_new = from_y * ones(1, nof_points);
end
out.pixels = interp2(xx_org,yy_org,out.pixels,x_new,y_new);

% Construct a new axis
out.axis = out.axis(1);
out.axis.name = 'Sample Fine(xy)';
out.axis.count = length(out.pixels);
out.axis.direction = '?';
out.axis.interval = 'NaN';
out.axis.variable(1).ref_index = ix;
out.axis.variable(2).ref_index = iy;

if strcmp(axis_type, 'none') % Pixel coordinates
    if nx > 1 % Otherwise something strange is going on, but
        % we are allowing empty channels for now
        out.parameter(ix).start_value.fine =  ...
            in.parameter(ix).start_value.fine + extent_x *...
            (from_x - 1)/ (nx - 1);
        out.parameter(ix).start_value.real =  ...
            in.parameter(ix).start_value.real + extent_x *...
            (from_x - 1)/ (nx - 1);
        out.axis.variable(1).extent = ...
            abs(in.axis(1).variable.extent) * (to_x - from_x) / (nx - 1);
    end
    if ny > 1 % Otherwise something strange is going on, but
        % we are allowing empty channels for now
        out.parameter(iy).start_value.fine =  ...
            in.parameter(iy).start_value.fine + extent_y *...
            (from_y - 1)/ (ny - 1);
        out.parameter(iy).start_value.real =  ...
            in.parameter(iy).start_value.real + extent_y *...
            (from_y - 1)/ (ny - 1);
        
        out.axis.variable(2).extent =  ...
            abs(in.axis(2).variable.extent) * (to_y - from_y) / (ny - 1);
    end
else % Real-world coordinates
    if scale_x > 0 && scale_y > 0
        out.parameter(ix).start_value.fine = ...
            in.parameter(ix).start_value.fine + ...
            (from_x - hor_min) / scale_x ;
        out.axis.variable(1).extent = (to_x - from_x) / scale_x;
        out.parameter(iy).start_value.fine = ...
            in.parameter(iy).start_value.fine + ...
            (from_y - vert_min) / scale_y ;
        out.axis.variable(2).extent = (to_y - from_y) / scale_y;
    else
        me = MException('spm:cut:invalid_display_scale', ...
            'Negative display scales are not allowed.');
        throw(me);
    end
end

% Change channel type
out.type = 'generalScanStandard';

end


