function out = spm_flatten(in, varargin)
%SPM_FLATTEN Flattening of the pixel matrix of a channel.
%   SPM_FLATTEN(IN, MASK, ORDER, MODE) with MODE = 'global' considers the 
%   pixel matrix of the channel IN as a function of the index grid and  
%   subtracts the best fitting polynomial of order ORDER using only those 
%   points that are labeled with ones in the binary mask MASK to compute 
%   the fit. With MODE = 'line', each line of the image is treated 
%   separately. The arguments MASK, ORDER and MODE are optional, with 
%   default values the all-one matrix, 0 and 'global', resp.

% Check input
try
    parser = inputParser;
    parser.addRequired('in', @is_channel);
    parser.addOptional('mask', in, ...
        @(mask)is_channel(mask) && length(find(mask.pixels == 1)) + ...
        length(find(mask.pixels == 0)) ==  numel(mask.pixels));
    parser.addOptional('order', 0,  @(order)(isnumeric(order) && ...
        numel(order) == 1 && floor(order) == order) && order >= 0);
    parser.addOptional('mode', 'global', @(mode)(ischar(mode) & ...
        (strcmp(mode,'global') || strcmp(mode,'line'))));
    parser.parse(in, varargin{:});
    mask = parser.Results.mask;
    if nargin == 1
        mask.pixels = ones(size(mask.pixels));
    end
    mode = parser.Results.mode;
    order = parser.Results.order;
catch error
    me = MException('spm:spm_flatten:input_error', ...
        'Invalid input: %s', error.message);
    throw(me);
end 

if size(mask) ~= size(in)
    me = MException('spm:spm_flatten:unequal_dimensions', ...
        'Dimensions of input matrix and mask do not match.');
    throw(me);
end

% Carry out fit and subtraction
out = in;
try
    pixels = subtract_polynom(in.pixels, mask.pixels, order, mode);
catch error
     me = MException('spm:spm_flatten:fitting_error', ...
        'Polynom subtraction failed. %s', error.message);
    throw(me);
end
out.pixels = pixels;

end

