function out = spm_shift(in, dx, dy)
%SPM_SHIFT Apply a (circular) shift.
%   SPM_SHIFT(IN, DX, DY) shifts the pixel matrix of the spatial imageScan 
%   channel IN over DX x DY pixels, where pixels ending up outside the 
%   border of the frame are wrapped around at the opposite side (circular 
%   shifting). The axes limits of the channel and eventual shift parameters 
%   are adapted accordingly.

% Check input
try
    parser = inputParser;
    parser.addRequired('in', @(in)(is_channel(in)  && ...
        strcmp(in.type,'imageScan') && ...
        strcmp(in.pixel_domain,'spatial')));
    parser.addRequired('dx', @(dx)(isnumeric(dx) && numel(dx) == 1 && ...
        floor(dx) == dx));
    parser.addRequired('dy', @(dy)(isnumeric(dy) && numel(dy) == 1 && ...
        floor(dy) == dy));
    parser.parse(in, dx, dy);
catch error
    me = MException('spm:spm_shift:input_error', ...
        'Invalid input: %s', error.message);
    throw(me);
end

out = in;
out.pixels = circshift(out.pixels,[dy,dx]);
ix = out.axis(1).variable.ref_index;
iy = out.axis(2).variable.ref_index;

% Adapt axes limits
if out.axis(2).count > 0 % Otherwise shift should be zero
    y_shift = dy * abs(out.axis(2).variable.extent) / ...
       out.axis(2).count;
end

if out.axis(1).count > 0 % Otherwise shift should be zero
    x_shift =  dx * abs(out.axis(1).variable.extent) / ...
       out.axis(1).count;
end

out.parameter(ix).start_value.fine = ...
    out.parameter(ix).start_value.fine - x_shift;
out.parameter(iy).start_value.fine = ...
    out.parameter(iy).start_value.fine - y_shift;

out.parameter(ix).start_value.real = ...
    out.parameter(ix).start_value.real - x_shift;
out.parameter(iy).start_value.real = ...
    out.parameter(iy).start_value.real - y_shift;

% Adapt shift values
out.parameter(ix).shift = out.parameter(ix).shift - x_shift;
out.parameter(iy).shift = out.parameter(iy).shift - y_shift;

end

