function spm_write(channel, output_path, xml_name, varargin)
%SPM_WRITE Write a channel to an XML file.
%   spm_write(CHANNEL, OUTPUT_PATH, XML_NAME) writes the channel CHANNEL
%   to the EMPA XML file XML_NAME and saves this file and the session
%   history to the directory OUTPUT_PATH. The name of the session history
%   file is automatically set to the session start date/time.
%
%   When calling spm_write(CHANNEL, OUTPUT_PATH) then the CHANNEL variable
%   name is taken as the name of the output xml and session file with
%   extension .xml and .m, respectively. If CHANNEL is given by
%   an expression, rather than a variable then the current date in ISO 8601
%   'yyyymmddTHHMMSS'- format is used as a name. Similarly,
%   spm_write(CHANNEL,OUTPUT_PATH,XML_NAME) constructs a default name for
%   the session file.
%
%   Upon writing, the context files are all copied to OUTPUT_PATH and all
%   path prefixes are stripped from the context array before storing the
%   context to file.
%
%   If the desired output format is a NanoScan XML file, then this may be
%   indicated by an optional property-value pair, where the property name
%   is 'type' with possible values 'nanoscan' and 'empa'. In this case, the
%   filename needs to be explicily specified. So spm_write(CHANNEL,
%   OUTPUT_PATH, XML_NAME) is actually equivalent to  spm_write(CHANNEL,
%   OUTPUT_PATH, XML_NAME, 'type', 'empa').
%
%   If 'type' is set to 'nanoscan', then one will need to fill up NanoScan
%   XML elements that are not part of the CHANNEL structure. For this
%   purpose, one can set the property 'template' to indicate a path to this
%   file. So a call like spm_write(CHANNEL, OUTPUT_PATH, XML_NAME, 'type',
%   'nanoscan', 'template', '/home/username/template.xml') or
%   spm_write(CHANNEL, OUTPUT_PATH, XML_NAME, 'template',
%   'C:\Temp\template.xml') is also valid. Note that the type can be
%   omitted if a template is specified as EMPA XML-files, as these do not
%   require templates.
%
%   IF 'template' is set to the empty string, then the empty template is
%   used (see the file trunk/XML/NanoScan/spmscan.xml). If 'template' is
%   omitted altogether (and 'type' is set to 'nanoscan'), then the first
%   context file of the channel is used if this is an XML file. Otherwise,
%   the empty template is used.

% Check input
full_path = mfilename('fullpath');
root_path = fileparts(full_path);
template_path =[root_path, filesep, 'xml' filesep];

if verLessThan('matlab','7.14')
    error(nargchk(2,7,nargin)); %#ok<NCHKN>
else
    narginchk(2,7);
end

if ~is_channel(channel)
    me = MException('spm:spm_write:no_channel_provided', ...
        'First argument should be a channel.');
    throw(me);
else
    if ~ischar(output_path)
        me = MException('spm:spm_write:no_output_path_provided', ...
            'Second argument should be a string.');
        throw(me);
    end
end

type = 'empa';
template = '';
if nargin < 3
    if ~isempty(inputname(1))
        name = inputname(1);
    else
        function_call = call_string();
        function_call = strtrim(function_call);
        if function_call(end) == ';'
            function_call = function_call(1:end - 1);
        end
        function_call = strtrim(function_call);
        left_bracket = strfind(function_call,'(');
        
        if ~isempty(left_bracket)
            start = left_bracket(1) + 1;
            for i=start:length(function_call)
                if function_call(i) == ','
                    try
                        evalin('caller', [function_call(start:i-1),';']);
                    catch
                        continue;
                    end
                    break;
                end
            end
            change_name = true;
            if i == length(function_call)
                try
                    evalin('caller', [function_call(start:i-1),';'])
                catch
                    change_name = false;
                end
            end
            if change_name
                new_name = strtrim(function_call(start:i-1));
                fid = fopen([tempdir, new_name], 'w');
                if fid ~= -1
                    fclose(fid);
                    delete([tempdir, new_name]);
                    name = new_name;
                else
                    warning('spm:spm_write:simple_auto_name', ...
                        ['Input channel name does not seem to be ' , ...
                        'a valid filename. Resorting to using date ' , ...
                        'as filename'] );
                    name = datestr(now,'yyyymmddTHHMMSS');
                end
            end
        end
    end
    xml_name = [name,'.xml'];
else
    if (nargin == 5 || nargin == 7)
        if (strcmp(varargin{1},'template'))
            template = varargin{2};
        else
            if (strcmp(varargin{1},'type') )
                type = varargin{2};
            end
        end
        if (nargin == 7)
            if (strmp(varargin{3},'template'))
                template = varargin{4};
            end
            if (strcmp(varargin{3},'type') )
                type = varargin{4};
            end
        end
    else
        if nargin ~= 3
             me = MException('spm:spm_write:input_error', ...
            ['Unexpected number of input parameters. ', ...
            'Please review function call.']);
             throw(me);
        end
    end
end

if (strcmp(template, '') && strcmp(type, 'nanoscan'))
    if ~isempty(channel.context)
        [~,~,ext] = fileparts(channel.context{1});
        if strcmp(ext,'.xml')
            template = channel.context{1};
        else
            template = [template_path,'spmscan.xml'];
        end
    else
        template = [template_path,'spmscan.xml'];
    end
end

if ~ismember(type, {'empa','nanoscan'})
    me = MException('spm:spm_write:unknown_file_type', ...
        'Unknown output file type.');
    throw(me);
end

if ~ischar(xml_name)
    me = MException('spm:spm_write:no_xml_name_provided', ...
        'Second argument should be a string.');
    throw(me)
else
    if ~ischar(output_path)
        me = MException('spm:spm_write:no_output_path_provided', ...
            'Third argument should be a string.');
        throw(me);
    end
end

% Create output path if it does not exist
if ~exist(output_path,'dir')
    mkdir(output_path);
end

% Save session
if ~isdeployed
    [session_file, ~] = preserve_session();
    fid_out = fopen(get_full_path(fullfile(output_path,session_file)),'w');
    fid_in = fopen(get_full_path(fullfile(prefdir,session_file)),'r');
    i=0;
    while ~feof(fid_in)
        i=i+1;
        line = fgets(fid_in);
        fprintf(fid_out,'%s',line);
    end
    fclose(fid_in);
    fclose(fid_out);
else
    warning('spm:spm_write:no_session_saved', ...
        'Session file is not saved (not compatible with deployed mode)');
    session_file = '';
end

% Copy context files and convert each context to a relative filename
for i=1:length(channel.context)
    [~,name,ext] = fileparts(channel.context{i});
    if ~strcmp(get_full_path(channel.context{i}),...
            get_full_path(fullfile(output_path,[name,ext]))) % No self-copy
        [status, message] = copyfile(channel.context{i},output_path);
        if ~status
            warning('spm:spm_write:context_copy_failed', ...
                'Could not copy context file %s.\n %s', ...
                channel.context{i},message);
        end
    end
    [~,name,ext] = fileparts(channel.context{i});
    channel.context{i} = [name,ext];
end

% Convert pixel matrix to vector and base64encode this vector
channel.pixels = channel.pixels';
% Pure real-valued pixel data
if strcmp(channel.pixel_type,'real')
    channel.pixels = channel.pixels(:);
    % Complex pixel data
elseif strcmp(channel.pixel_type,'complex')
    channel.pixels = [real(channel.pixels(:));imag(channel.pixels(:))];
    % Unknow pixel data type
else
    me = MException('spm:spm_write:unknown_pixel_type', ...
        'Unknown pixel type.');
    throw(me);
end

channel.pixels = single(channel.pixels);
[~,~,endian] = computer;
if endian == 'B'
    channel.pixels = swapbytes(channel.pixels);
end
channel.pixels = typecast(single(channel.pixels),'uint8');
channel.pixels = base64encode(channel.pixels);

% Add history string to struct
channel.session = session_file;

% Convert class object to struct and put fields in correct order
empa_skeleton_file = fullfile(root_path,  'xml', 'empa_skeleton.xml');
empa_skeleton = xml_read(empa_skeleton_file);
if ~isfield(channel,'control')
    empa_skeleton.channel = rmfield(empa_skeleton.channel, 'control');
end
id = 'MATLAB:structOnObject';
warning('off', id);
channel = struct(channel);
if isfield(channel,'control') && isempty(channel.control)
	channel = rmfield(channel,'control');
end
warning('on', id);
channel = orderfields(channel,empa_skeleton.channel);

% Construct channel array containing channel (EMPA->NanoScan stylesheet
% expects an array as an input
channel_list.channel = channel;

if strcmp(type, 'empa') % EMPA XML
    pref.StructItem = false;
    pref.CellItem = false;
    xml_write(get_full_path(fullfile(output_path,xml_name)), ...
        channel_list, 'channel_list', pref);
else % NanoScan XML
    % Apply EMPA->NanoScan stylesheet to create a NanoScan XML barebone from
    % the structure
    pref.StructItem = false;
    pref.CellItem = false;
    if exist([template_path, 'empa2nanoscan.xsl'], 'file')
        empa2nanoscan_stylesheet = [template_path, 'empa2nanoscan.xsl'];
        compiled = false;
    elseif  exist([template_path, 'empa2nanoscan.sxx'],'file')
        empa2nanoscan_stylesheet = [template_path, 'empa2nanoscan.sxx'];
        compiled = true;
    else
        me = MException('spm:spm_write:missing_stylesheet', ...
            'Cannot find EMPA_>NanoScan conversion stylesheet.');
        throw(me);
    end
    dom_empa = xml_write([],channel_list,'channel_list',pref);
    barebone =  xslt_flex(dom_empa,empa2nanoscan_stylesheet, ...
            'saxon', compiled);
    
    %Read the empty NanoScan file
    empty_tree = xmlread([template_path, 'empty_spmscan.xml']);
    
    % Read the NanoScan template file
    template_tree = xmlread(template);
    
    % Cut the axis/area element from the template tree
    axis_nodes = template_tree.getElementsByTagName('axis');
    axis_element= axis_nodes.item(0);
    if ~isempty(axis_element)
        axis_element.getParentNode().removeChild(axis_element);
    else
        area_nodes = template_tree.getElementsByTagName('area');
        area_element= area_nodes.item(0);
        if ~isempty(area_element)
            area_element.getParentNode().removeChild(area_element);
        end
    end
    
    % First remove all attributes from the root to prepare for a merge
    template_scan_elements = template_tree.getElementsByTagName('scan');
    template_scan_element = template_scan_elements.item(0);
    remove_all_attributes(template_scan_element);
    
    barebone_scan_elements = barebone.getElementsByTagName('scan');
    barebone_scan_element = barebone_scan_elements.item(0);
    remove_all_attributes(barebone_scan_element);
    
    % Merge the empty NanoScan tree with the template tree
    default_merge_properties = [template_path, 'default_merge.properties'];
    extended_template = dom_merge(empty_tree, template_tree, ...
        default_merge_properties);
    
    % Merge the extended template tree with the NanoScan barebone into an
    % (extended) NanoScan XML string
    nanoscan_merge_properties = ...
        [template_path, 'nanoscan_merge.properties'];
    merged = dom_merge(extended_template, barebone, ...
        nanoscan_merge_properties);
    
    % Recursively strip empty elements from extended NanoScan XML string
    clear_empty_stylesheet = [template_path, 'clear_empty.xsl'];
    clean_merged = xslt_flex(merged, clear_empty_stylesheet, 'saxon');
    
    clean_merged_scan_elements = clean_merged.getElementsByTagName('scan');
    clean_merged_scan_element = clean_merged_scan_elements.item(0);
    
    clean_merged_scan_element.setAttribute('version', '2.0');
    clean_merged_scan_element.setAttribute('xmlns', ...
        'http://www.nanoscan.ch/SPM');
    clean_merged_scan_element.setAttribute('xmlns:xsi', ...
        'http://www.w3.org/2001/XMLSchema-instance');
    clean_merged_scan_element.setAttribute('xsi:schemaLocation', ...
        'http://www.nanoscan.ch/SPM spmscan.xsd');
    
    % Save the XML string to file
    xmlwrite_xerces(get_full_path(fullfile(output_path,xml_name)), ...
        clean_merged);
end

end

