function out = threshold(in, varargin)
%THRESHOLD Binarization using an interactively selected level.
%   THRESHOLD(IN) returns the result of leveling IN at a user-selected
%   binarization level. The user selects the level by moving a slider or
%   editing a text box and closes the figure window when finished.
%
%   THRESHOLD(IN, LEVEL) levels without user interaction at level
%   LEVEL.

try
    parser = inputParser;
    parser.addRequired('in', @(in)(is_channel(in)  && ...
        strcmp(in.type,'imageScan') && ...
        strcmp(in.pixel_domain,'spatial')));
    parser.addOptional('level', NaN, @(level)(...
        numel(level) == 1 && isnumeric(level)))
    parser.parse(in, varargin{:});
    level = parser.Results.level;
catch error
    me = MException('spm:level:input_error', ...
        'Invalid input: %s', error.message);
    throw(me);
end

% Get dimensions of channel
[x_min, x_max, y_min, y_max] = get_axes_limits(in,'relative','relative');
x_size = x_max - x_min;
y_size = y_max- y_min;
nx = size(in.pixels,2);
ny = size(in.pixels,1);


if isnan(level)
    spm_display(in, 'window_name', 'threshold gui');
    fig = gcf;
    

    binary = zeros(ny, nx);
    hold on;

    overlay = subimage(255 * ones(size(in.pixels)),[0,0,0; 1 0 0]);
    set(overlay, 'AlphaData', zeros(size(in.pixels)));
    max_pix = max(in.pixels(:));
    min_pix = min(in.pixels(:));
    level = max_pix;
    
    % Get position of color bar
    cbar = findobj(fig,'tag','Colorbar');
    cbar_units = get(cbar,'Units');
    set(cbar,'Units', 'pixels');
    pos_cbar = get(cbar,'Position');
    
    % Set position of all axes to pixels (no auto-resize)
    ax = findall(fig, 'type', 'axes');
    unit_ax = cell(length(ax), 1);
    pos_ax = zeros(length(ax), 4);
    for k = 1:length(ax)
        if ax(k) ~= cbar
            unit_ax{k} = get(ax(k), 'Units');
            set(ax(k), 'Units', 'pixels');
            pos_ax(k, :) = get(ax(k), 'Position');
        else
            unit_ax{k} =  cbar_units;
            pos_ax(k, :) = pos_cbar;
        end
    end
    
    % Adapt figure position to create room for slider and edit box
    if true % Workaround for Matlab bug (ticket:129).
        delete(cbar);
         warning('spm:threshold:removed_colorbar', ...
            ['Cannot show colorbar due to a known ', ...
             'Matlab rendering bug. \n', ...
             'Issue will be solved in future Matlab versions.']);
    end
    unit_fig = get(fig, 'Units');
    set(fig, 'Units', 'pixels');
    pos_fig = get(fig, 'position');
    set(fig, 'Position', [pos_fig(1) ,pos_fig(2), ...
        pos_fig(3) + 2 * pos_cbar(3), pos_fig(4)]);
    set(fig, 'Units', unit_fig);
    
    % Place slider and edit box
    height =  max(pos_ax(:,4));
    y = min(pos_ax(:,2));
    if  true% Workaround form Matlab bug (ticket:129).
        slider = uicontrol('Style','slider','Units','pixels', ...
            'Position',  ...
            [pos_fig(3) - 1.75 * pos_cbar(3), y, pos_cbar(3), height],...
            'Value',1);
        edit_box = uicontrol('Style','edit','Units','pixels', ...
            'Position',...
            [pos_fig(3) - 2.5 * pos_cbar(3), y - 1.1 *pos_cbar(3), ...
            2.5 * pos_cbar(3), pos_cbar(3)], ...
            'String', num2str(level,'%0.4g'), 'Callback',@edit);
    else
        slider = uicontrol('Style','slider','Units','pixels', ...
            'Position',...
            [pos_fig(3) + 0.5 * pos_cbar(3), y, pos_cbar(3), ...
            height],...
            'Value',1);
        edit_box = uicontrol('Style','edit','Units','pixels', ...
            'Position',...
            [pos_fig(3) - pos_cbar(3), y - 1.1 *pos_cbar(3), ...
            2.5 * pos_cbar(3), pos_cbar(3)], ...
            'String', num2str(level,'%0.4g'), 'Callback',@edit);
    end
    set(slider, 'Units', 'normalized');
    set(edit_box, 'Units', 'normalized');
    
    if ~verLessThan('matlab', '7.12.0')
        addlistener(slider,'ContinuousValueChange',@slide);
    else
        set(slider,'CallBack',@slide);
    end
    set(fig,'CloseRequestFcn', {@close_fig});
    key_press_fcn =  get(fig, 'KeyPressFcn');
    set(fig,'KeyPressFcn',@finish);
    
    % Set axes units back to original values
    for k = 1:length(ax)
        if ishandle(ax(k))
            set(ax(k), 'Units', unit_ax{k});
        end
    end
    
    uiwait(fig);
else
    binary = double(in.pixels >= level);
    get_out();
end
[session_file, line_number] = preserve_session();
if isempty(in.comment)
    out.comment = sprintf('line %d of %s : threshold level %f', ...
        line_number, session_file, level);
else
    out.comment = [in.comment, ...
        sprintf(', line %d of %s : threshold level %f', line_number, ...
        session_file, level)];
end

    function edit(~, ~)
        level = str2double(get(edit_box,'String'));
        slider_value  = (level - min_pix) / (max_pix - min_pix + ...
            eps(max_pix - min_pix));
        slider_value = max(min(slider_value, 1), 0);
        set(slider,'Value', slider_value);
        update_mask();
    end

    function slide(~, ~)
        slider_value = get(slider,'Value');
        level = min_pix + slider_value * (max_pix - min_pix + ...
            eps(max_pix - min_pix));
        set(edit_box,'String', num2str(level,'%0.4g'));
        update_mask();
    end

    function update_mask()
        binary = double(in.pixels >= level);
        set(overlay,'AlphaData', binary);
        set(findobj(fig, 'Type', 'uicontrol'), 'Enable', 'off');
        drawnow;
        set(findobj(fig, 'Type', 'uicontrol'), 'Enable', 'on');
    end

    function close_fig(~,~)
        uiresume(fig);
        get_out();
        delete(fig);
    end

    function get_out()
        out = empty_image(x_size, y_size, nx, ny, 'm');
        out.axis = in.axis;
        out.parameter = in.parameter;
        out.context = in.context;
        out.pixels = binary;
    end

    function finish(~,event)
        if strcmp(event.Key,'return')
            set(fig,'KeyPressFcn',key_press_fcn);
            set(edit_box,'Enable','off');
            set(slider,'Enable','off');
            uiresume(fig);
            get_out();
        end
    end
end